#include "stream.h"
#include <catch.h>
#include <vector>
#include <vector>

using namespace std;
using namespace stream;

TEST_CASE("take(10, iota(0))")
{
    auto stream = take(10, iota(0));
    vector<int> v = collect<vector<int>>(stream);
    vector<int> expected{0, 1, 2, 3, 4, 5, 6, 7, 8, 9};

    CHECK(expected == v);
}

TEST_CASE("range-based for over iota(0, 10)")
{
    vector<int> v;
    vector<int> expected{0, 1, 2, 3, 4, 5, 6, 7, 8, 9};

    for (int i : iota(0, 10))
        v.push_back(i);

    CHECK(expected == v);
}

TEST_CASE("map")
{
    auto stream = take(10, map(iota(0), [](auto elem) { return elem + 1; }));
    vector<int> v = collect<vector<int>>(stream);
    vector<int> expected{1, 2, 3, 4, 5, 6, 7, 8, 9, 10};

    CHECK(expected == v);
}

TEST_CASE("filter")
{
    auto stream = filter(iota(0, 10), [](auto elem) { return elem % 2 == 0; });
    vector<int> v = collect<vector<int>>(stream);
    vector<int> expected{0, 2, 4, 6, 8};

    CHECK(expected == v);
}
